@@ -21,26 +21,19 @@
 #ifndef DOCKCORONA_H
 #define DOCKCORONA_H
 
-#include "dockview.h"
-#include "globalshortcuts.h"
-#include "layoutmanager.h"
-#include "universalsettings.h"
 #include "../liblattedock/dock.h"
-#include "launcherssignals.h"
 
 #include <QObject>
+#include <QTimer>
+
+#include <Plasma/Corona>
+//#include <PlasmaQuick/ConfigView>
 
 #include <KAboutApplicationDialog>
-#include <KDeclarative/QmlObject>
+//#include <KDeclarative/QmlObject>
 
-class ScreenPool;
-class GlobalShortcuts;
-class UniversalSettings;
-class LayoutManager;
-class LaunchersSignals;
-
-namespace KActivities {
-class Consumer;
+namespace KDeclarative {
+class QmlObject;
 }
 
 namespace Plasma {
@@ -49,12 +42,29 @@ class Containment;
 class Types;
 }
 
+namespace PlasmaQuick {
+class ConfigView;
+}
+
+namespace KActivities {
+class Consumer;
+}
+
 namespace KWayland {
 namespace Client {
 class PlasmaShell;
 }
 }
 
+namespace Latte {
+class AbstractWindowInterface;
+class ScreenPool;
+class GlobalShortcuts;
+class UniversalSettings;
+class LayoutManager;
+class LaunchersSignals;
+}
+
 namespace Latte {
 
 class DockCorona : public Plasma::Corona {
@@ -62,7 +72,10 @@ class DockCorona : public Plasma::Corona {
     Q_CLASSINFO("D-Bus Interface", "org.kde.LatteDock")
 
 public:
-    DockCorona(QObject *parent = nullptr);
+    DockCorona(bool defaultLayoutOnStartup = false,
+               QString layoutNameOnStartUp = QString(),
+               int userSetMemoryUsage = -1,
+               QObject *parent = nullptr);
     virtual ~DockCorona();
 
     int numScreens() const override;
@@ -70,35 +83,36 @@ public:
     QRegion availableScreenRegion(int id) const override;
     QRect availableScreenRect(int id) const override;
 
-    QList<Plasma::Types::Location> freeEdges(int screen) const;
-    QList<Plasma::Types::Location> freeEdges(QScreen *screen) const;
+    //! This is a very generic function in order to return the availableScreenRect of specific screen
+    //! by calculating only the user specified visibility modes and edges. Empty QLists for both
+    //! arguments mean that all choices are accepted in calculations
+    QRect availableScreenRectWithCriteria(int id,
+                                          QList<Dock::Visibility> modes = QList<Dock::Visibility>(),
+                                          QList<Plasma::Types::Location> edges = QList<Plasma::Types::Location>()) const;
 
-    void loadLatteLayout(QString layoutPath);
+    QRegion availableScreenRegionWithCriteria(int id, QString forLayout = QString()) const;
 
-    int docksCount() const;
-    int docksCount(int screen) const;
-    int docksCount(QScreen *screen) const;
-
-    int noDocksWithTasks() const;
     int screenForContainment(const Plasma::Containment *containment) const override;
 
-    void addDock(Plasma::Containment *containment, int expDockScreen = -1);
-    void recreateDock(Plasma::Containment *containment);
-    void copyDock(Plasma::Containment *containment);
-
-    void aboutApplication();
     void closeApplication();
 
+    AbstractWindowInterface *wm() const;
+    KActivities::Consumer *activitiesConsumer() const;
     ScreenPool *screenPool() const;
     UniversalSettings *universalSettings() const;
     LayoutManager *layoutManager() const;
 
     KWayland::Client::PlasmaShell *waylandDockCoronaInterface() const;
 
+    //! these functions are used from context menu through containmentactions
+    void switchToLayout(QString layout);
+    void showSettingsWindow(int page);
+    QStringList contextMenuData();
+
 public slots:
+    void aboutApplication();
     void activateLauncherMenu();
     void loadDefaultLayout() override;
-    void dockContainmentDestroyed(QObject *cont);
     void updateDockItemBadge(QString identifier, QString value);
     void unload();
 
@@ -109,43 +123,38 @@ signals:
     void raiseDocksTemporaryChanged();
 
 private slots:
-    void destroyedChanged(bool destroyed);
-    void showAlternativesForApplet(Plasma::Applet *applet);
     void alternativesVisibilityChanged(bool visible);
+    void showAlternativesForApplet(Plasma::Applet *applet);
     void load();
 
     void addOutput(QScreen *screen);
     void primaryOutputChanged();
     void screenRemoved(QScreen *screen);
     void screenCountChanged();
-    void syncDockViews();
+    void syncDockViewsToScreens();
 
 private:
     void cleanConfig();
-    void cleanupOnStartup(QString path);
     void qmlRegisterTypes() const;
     void setupWaylandIntegration();
 
     bool appletExists(uint containmentId, uint appletId) const;
     bool containmentExists(uint id) const;
-    bool explicitDockOccupyEdge(int screen, Plasma::Types::Location location)const;
-    bool heuresticForLoadingDockWithTasks();
-    int noOfDocks();
+
     int primaryScreenId() const;
-    QString availableId(QStringList all, QStringList assigned, int base);
+
     QStringList containmentsIds();
     QStringList appletsIds();
 
     bool m_activitiesStarting{true};
-    //! this is used to check if a dock with tasks in it will be loaded on startup
-    bool m_tasksWillBeLoaded{false};
-    //! this is used to record the first dock having tasks in it. It is used
-    //! to specify which dock will be loaded on startup if a case that no "dock
-    //! with tasks" will be loaded otherwise. Currently the older one dock wins
-    int m_firstContainmentWithTasks{ -1};
+    bool m_defaultLayoutOnStartup{false}; //! this is used to enforce loading the default layout on startup
+    bool m_quitTimedEnded{false}; //! this is used on destructor in order to delay it and slide-out the docks
+
+    //!it can be used on startup to change memory usage from command line
+    int m_userSetMemoryUsage{ -1};
+
+    QString m_layoutNameOnStartUp;
 
-    QHash<const Plasma::Containment *, DockView *> m_dockViews;
-    QHash<const Plasma::Containment *, DockView *> m_waitingDockViews;
     QList<KDeclarative::QmlObject *> m_alternativesObjects;
 
     QTimer m_docksScreenSyncTimer;
@@ -153,6 +162,7 @@ private:
     KActivities::Consumer *m_activityConsumer;
     QPointer<KAboutApplicationDialog> aboutDialog;
 
+    AbstractWindowInterface *m_wm{nullptr};
     ScreenPool *m_screenPool{nullptr};
     GlobalShortcuts *m_globalShortcuts{nullptr};
     UniversalSettings *m_universalSettings{nullptr};
@@ -161,6 +171,7 @@ private:
     KWayland::Client::PlasmaShell *m_waylandDockCorona{nullptr};
 
     friend class GlobalShortcuts;
+    friend class Layout;
     friend class LayoutManager;
     friend class LaunchersSignals;
 };
