@@ -32,6 +32,8 @@
     #include <xcb/xcb_event.h>
 #endif
 
+namespace Latte {
+
 ScreenPool::ScreenPool(KSharedConfig::Ptr config, QObject *parent)
     : QObject(parent),
       m_configGroup(KConfigGroup(config, QStringLiteral("ScreenConnectors")))
@@ -99,7 +101,7 @@ void ScreenPool::reload(QString path)
 {
     QFile rcfile(QString(path + "/lattedockrc"));
 
-    if (rcfile.exists()){
+    if (rcfile.exists()) {
         qDebug() << "load screen connectors from ::: " << rcfile.fileName();
         KSharedConfigPtr newFile = KSharedConfig::openConfig(rcfile.fileName());
         m_configGroup = KConfigGroup(newFile, QStringLiteral("ScreenConnectors"));
@@ -110,6 +112,11 @@ void ScreenPool::reload(QString path)
 
 }
 
+int ScreenPool::primaryScreenId() const
+{
+    return id(qGuiApp->primaryScreen()->name());
+}
+
 QString ScreenPool::primaryConnector() const
 {
     return m_primaryConnector;
@@ -271,5 +278,6 @@ bool ScreenPool::nativeEventFilter(const QByteArray &eventType, void *message, l
     return false;
 }
 
+}
 
 #include "moc_screenpool.cpp"
