@@ -19,6 +19,8 @@
 */
 
 #include "xwindowinterface.h"
+#include "dock/dockview.h"
+#include "dock/screenedgeghostwindow.h"
 #include "../liblattedock/extras.h"
 
 #include <QDebug>
@@ -29,6 +31,8 @@
 #include <KWindowInfo>
 #include <NETWM>
 
+#include <xcb/xcb.h>
+
 namespace Latte {
 
 XWindowInterface::XWindowInterface(QObject *parent)
@@ -84,7 +88,6 @@ void XWindowInterface::setDockExtraFlags(QWindow &view)
     KWindowSystem::setType(view.winId(), NET::Dock);
     KWindowSystem::setState(view.winId(), NET::SkipTaskbar | NET::SkipPager);
     KWindowSystem::setOnAllDesktops(view.winId(), true);
-    KWindowSystem::setOnActivities(view.winId(), {"0"});
 }
 
 void XWindowInterface::setDockStruts(QWindow &view, const QRect &rect
@@ -113,6 +116,7 @@ void XWindowInterface::setDockStruts(QWindow &view, const QRect &rect
             strut.bottom_end = rect.x() + rect.width() - 1;
             break;
         }
+
         case Plasma::Types::LeftEdge: {
             const int leftOffset = {screen->geometry().left()};
             strut.left_width = rect.width() + leftOffset;
@@ -120,6 +124,7 @@ void XWindowInterface::setDockStruts(QWindow &view, const QRect &rect
             strut.left_end = rect.y() + rect.height() - 1;
             break;
         }
+
         case Plasma::Types::RightEdge: {
             const int rightOffset = {wholeScreen.right() - currentScreen.right()};
             strut.right_width = rect.width() + rightOffset;
@@ -127,6 +132,7 @@ void XWindowInterface::setDockStruts(QWindow &view, const QRect &rect
             strut.right_end = rect.y() + rect.height() - 1;
             break;
         }
+
         default:
             qWarning() << "wrong location:" << qEnumToStr(location);
             return;
@@ -140,6 +146,11 @@ void XWindowInterface::setDockStruts(QWindow &view, const QRect &rect
                                    );
 }
 
+void XWindowInterface::setWindowOnActivities(QWindow &window, const QStringList &activities)
+{
+    KWindowSystem::setOnActivities(window.winId(), activities);
+}
+
 void XWindowInterface::removeDockStruts(QWindow &view) const
 {
     KWindowSystem::setStrut(view.winId(), 0, 0, 0, 0);
@@ -155,6 +166,15 @@ const std::list<WindowId> &XWindowInterface::windows() const
     return m_windows;
 }
 
+void XWindowInterface::setKeepAbove(const QDialog &dialog, bool above) const
+{
+    if (above) {
+        KWindowSystem::setState(dialog.winId(), NET::KeepAbove);
+    } else {
+        KWindowSystem::clearState(dialog.winId(), NET::KeepAbove);
+    }
+}
+
 void XWindowInterface::skipTaskBar(const QDialog &dialog) const
 {
     KWindowSystem::setState(dialog.winId(), NET::SkipTaskbar);
@@ -193,6 +213,65 @@ void XWindowInterface::enableBlurBehind(QWindow &view) const
     KWindowEffects::enableBlurBehind(view.winId());
 }
 
+void XWindowInterface::setEdgeStateFor(QWindow *view, bool active) const
+{
+    ScreenEdgeGhostWindow *window = qobject_cast<ScreenEdgeGhostWindow *>(view);
+
+    if (!window) {
+        return;
+    }
+
+    xcb_connection_t *c = QX11Info::connection();
+
+    const QByteArray effectName = QByteArrayLiteral("_KDE_NET_WM_SCREEN_EDGE_SHOW");
+    xcb_intern_atom_cookie_t atomCookie = xcb_intern_atom_unchecked(c, false, effectName.length(), effectName.constData());
+
+    QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atom(xcb_intern_atom_reply(c, atomCookie, nullptr));
+
+    if (!atom) {
+        return;
+    }
+
+    if (!active) {
+        xcb_delete_property(c, window->winId(), atom->atom);
+        window->hideWithMask();
+        return;
+    }
+
+    window->showWithMask();
+
+    uint32_t value = 0;
+
+    switch (window->location()) {
+        case Plasma::Types::TopEdge:
+            value = 0;
+            break;
+
+        case Plasma::Types::RightEdge:
+            value = 1;
+            break;
+
+        case Plasma::Types::BottomEdge:
+            value = 2;
+            break;
+
+        case Plasma::Types::LeftEdge:
+            value = 3;
+            break;
+
+        case Plasma::Types::Floating:
+        default:
+            value = 4;
+            break;
+    }
+
+    int hideType = 0;
+
+    value |= hideType << 8;
+
+    xcb_change_property(c, XCB_PROP_MODE_REPLACE, window->winId(), atom->atom, XCB_ATOM_CARDINAL, 32, 1, &value);
+}
+
 WindowInfoWrap XWindowInterface::requestInfoActive() const
 {
     return requestInfo(KWindowSystem::activeWindow());
@@ -209,7 +288,7 @@ bool XWindowInterface::isOnCurrentActivity(WindowId wid) const
     KWindowInfo winfo(wid.value<WId>(), 0, NET::WM2Activities);
 
     return winfo.valid()
-            && (winfo.activities().contains(m_activities->currentActivity()) || winfo.activities().empty());
+           && (winfo.activities().contains(m_activities->currentActivity()) || winfo.activities().empty());
 }
 
 WindowInfoWrap XWindowInterface::requestInfo(WindowId wid) const
@@ -231,6 +310,7 @@ WindowInfoWrap XWindowInterface::requestInfo(WindowId wid) const
         winfoWrap.setIsFullscreen(winfo.hasState(NET::FullScreen));
         winfoWrap.setIsShaded(winfo.hasState(NET::Shaded));
         winfoWrap.setGeometry(winfo.frameGeometry());
+        winfoWrap.setIsKeepAbove(winfo.hasState(NET::KeepAbove));
     } else if (m_desktopId == wid) {
         winfoWrap.setIsValid(true);
         winfoWrap.setIsPlasmaDesktop(true);
